/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2023 Sony Corporation. All rights reserved.
 */
#ifndef IFRTP_DRIVER_H_
#define IFRTP_DRIVER_H_
#include <linux/netdevice.h>
#include <linux/irqreturn.h>
#include <linux/inetdevice.h>
#include <net/route.h>
#include "ifrtp_ioctl.h"
#include "ifrtp_address.h"

#define IFRTP_NUMVFS_MAX	24

#define RTP_SSRC_MAX		28
#define RTP_SSRC_PER_CH		3
#define RTP_QFHD_CH_PER_ST	4
#define RTP_MIN_PACKET_SIZE	12
#define RTP_MAX_PACKET_SIZE	(ETH_DATA_LEN - 28)
#define RTP_PACKET_OFFSET_SIZE	1536

#define IFRTP_SDI_CH_MAX_PER_CORE	8

#define IFRTP_ARP_SCHEDULE_INTERVAL	120

enum ifrtp_interface_type {
	IFRTP_IF_IVFRTP,
	IFRTP_IF_IGW,
	IFRTP_IF_ERROR
};

struct ifrtp_recv_mem {
	uintptr_t addr;
	uintptr_t begin;
	uintptr_t end;
	uintptr_t begin_offset;
};

struct ifrtp_priv {
	struct rtnl_link_stats64 stats;

	spinlock_t *tx_desc_lock;
	u32 *desc_addr_tx;
	u32 *desc_addr_rx;

	struct ifrtp_recv_mem *recv_buff;
	u32 ssrc_count[RTP_SSRC_PER_CH];
	unsigned long ssrc_mismatch_last;

	struct net_device *dev;
	u32 stream_id;
	u64 saddr;
	unsigned int stream_mode;
	bool is_main_mac;
	bool stream_tx_ready;
};

struct ifrtp_packet_count {
	u32 packet_count[16];
	u32 packet_size[16];
};

int ifrtp_init_driver(struct platform_device *pdev);
int ifrtp_init_netlink(struct net_device *dev);
void ifrtp_cleanup_driver(void);
irqreturn_t ifrtp_napi_schedule(void);
int arp_query(__be32 paddr, int igw_if_id, char *eth_dst);
void ifrtp_init_rtp_recv_buffer(struct ifrtp_priv *priv);
int ifrtp_send_direct_multi_message(unsigned char *data, unsigned int len, unsigned int packet_type, struct sk_buff *skb);

#endif  // IFRTP_DRIVER_H_
